@@ -1,19 +1,24 @@
-import gevent
-from gevent import monkey, GreenletExit
-
-monkey.patch_all(thread=False)
-
-from time import time
-import sys
-import random
-import warnings
-import traceback
 import logging
+import random
+import sys
+import traceback
+from time import time
 
-from clients import HttpSession
-import events
+import gevent
+import six
 
-from exception import LocustError, InterruptTaskSet, RescheduleTask, RescheduleTaskImmediately, StopLocust
+from gevent import GreenletExit, monkey
+from six.moves import xrange
+
+# The monkey patching must run before requests is imported, or else 
+# we'll get an infinite recursion when doing SSL/HTTPS requests.
+# See: https://github.com/requests/requests/issues/3752#issuecomment-294608002
+monkey.patch_all()
+
+from . import events
+from .clients import HttpSession
+from .exception import (InterruptTaskSet, LocustError, RescheduleTask,
+                        RescheduleTaskImmediately, StopLocust)
 
 logger = logging.getLogger(__name__)
 
@@ -103,7 +108,7 @@ class Locust(object):
         except StopLocust:
             pass
         except (RescheduleTask, RescheduleTaskImmediately) as e:
-            raise LocustError, LocustError("A task inside a Locust class' main TaskSet (`%s.task_set` of type `%s`) seems to have called interrupt() or raised an InterruptTaskSet exception. The interrupt() function is used to hand over execution to a parent TaskSet, and should never be called in the main TaskSet which a Locust class' task_set attribute points to." % (type(self).__name__, self.task_set.__name__)), sys.exc_info()[2]
+            six.reraise(LocustError, LocustError("A task inside a Locust class' main TaskSet (`%s.task_set` of type `%s`) seems to have called interrupt() or raised an InterruptTaskSet exception. The interrupt() function is used to hand over execution to a parent TaskSet, and should never be called in the main TaskSet which a Locust class' task_set attribute points to." % (type(self).__name__, self.task_set.__name__)), sys.exc_info()[2])
 
 
 class HttpLocust(Locust):
@@ -146,7 +151,7 @@ class TaskSetMeta(type):
         if "tasks" in classDict and classDict["tasks"] is not None:
             tasks = classDict["tasks"]
             if isinstance(tasks, dict):
-                tasks = list(tasks.iteritems())
+                tasks = six.iteritems(tasks)
             
             for task in tasks:
                 if isinstance(task, tuple):
@@ -156,7 +161,7 @@ class TaskSetMeta(type):
                 else:
                     new_tasks.append(task)
         
-        for item in classDict.itervalues():
+        for item in six.itervalues(classDict):
             if hasattr(item, "locust_task_weight"):
                 for i in xrange(0, item.locust_task_weight):
                     new_tasks.append(item)
@@ -165,6 +170,7 @@ class TaskSetMeta(type):
         
         return type.__new__(mcs, classname, bases, classDict)
 
+@six.add_metaclass(TaskSetMeta)
 class TaskSet(object):
     """
     Class defining a set of tasks that a Locust user will execute. 
@@ -174,9 +180,9 @@ class TaskSet(object):
     *min_wait* and *max_wait* milliseconds. It will then schedule another task for 
     execution and so on.
     
-    TaskTests can be nested, which means that a TaskSet's *tasks* attribute can contain 
+    TaskSets can be nested, which means that a TaskSet's *tasks* attribute can contain 
     another TaskSet. If the nested TaskSet it scheduled to be executed, it will be 
-    instantiated and called from the current executing TaskSet. Execution in the the
+    instantiated and called from the current executing TaskSet. Execution in the
     currently running TaskSet will then be handed over to the nested TaskSet which will 
     continue to run until it throws an InterruptTaskSet exception, which is done when 
     :py:meth:`TaskSet.interrupt() <locust.core.TaskSet.interrupt>` is called. (execution 
@@ -221,8 +227,6 @@ class TaskSet(object):
     instantiated. Useful for nested TaskSet classes.
     """
 
-    __metaclass__ = TaskSetMeta    
-    
     def __init__(self, parent):
         self._task_queue = []
         self._time_start = time()
@@ -251,9 +255,9 @@ class TaskSet(object):
                 self.on_start()
         except InterruptTaskSet as e:
             if e.reschedule:
-                raise RescheduleTaskImmediately, e, sys.exc_info()[2]
+                six.reraise(RescheduleTaskImmediately, RescheduleTaskImmediately(e.reschedule), sys.exc_info()[2])
             else:
-                raise RescheduleTask, e, sys.exc_info()[2]
+                six.reraise(RescheduleTask, RescheduleTask(e.reschedule), sys.exc_info()[2])
         
         while (True):
             try:
@@ -273,9 +277,9 @@ class TaskSet(object):
                     self.wait()
             except InterruptTaskSet as e:
                 if e.reschedule:
-                    raise RescheduleTaskImmediately, e, sys.exc_info()[2]
+                    six.reraise(RescheduleTaskImmediately, RescheduleTaskImmediately(e.reschedule), sys.exc_info()[2])
                 else:
-                    raise RescheduleTask, e, sys.exc_info()[2]
+                    six.reraise(RescheduleTask, RescheduleTask(e.reschedule), sys.exc_info()[2])
             except StopLocust:
                 raise
             except GreenletExit:
@@ -294,7 +298,7 @@ class TaskSet(object):
     
     def execute_task(self, task, *args, **kwargs):
         # check if the function is a method bound to the current locust, and if so, don't pass self as first argument
-        if hasattr(task, "im_self") and task.__self__ == self:
+        if hasattr(task, "__self__") and task.__self__ == self:
             # task is a bound method on self
             task(*args, **kwargs)
         elif hasattr(task, "tasks") and issubclass(task, TaskSet):
@@ -352,4 +356,3 @@ class TaskSet(object):
         Locust instance.
         """
         return self.locust.client
-
